@@ -2,8 +2,12 @@
 # s/\(\w\+\)\s\+\(-\d\+\)/class \1(SystemException):\r    code = \2/g
 
 
-from __future__ import absolute_import
-import six
+import errno
+import numbers
+import os
+import sys
+
+
 class PycommandsException(Exception):
     pass
 
@@ -16,6 +20,10 @@ class DoesNotExist(PycommandsException):
     pass
 
 
+class InvalidInputArgument(PycommandsException):
+    pass
+
+
 class DataObjectDoesNotExist(DoesNotExist):
     pass
 
@@ -24,14 +32,22 @@ class CollectionDoesNotExist(DoesNotExist):
     pass
 
 
+class ZoneDoesNotExist(DoesNotExist):
+    pass
+
+
 class UserDoesNotExist(DoesNotExist):
     pass
 
 
-class UserGroupDoesNotExist(DoesNotExist):
+class GroupDoesNotExist(DoesNotExist):
     pass
 
 
+# NOTE: Everything of the form *UserGroup* is deprecated.
+UserGroupDoesNotExist = GroupDoesNotExist
+
+
 class ResourceDoesNotExist(DoesNotExist):
     pass
 
@@ -52,20 +68,157 @@ class MultipleResultsFound(QueryException):
     pass
 
 
+class NotImplementedInIRODSServer(PycommandsException):
+    def __init__(self, feature_description, required_iRODS_version=()):
+        super(NotImplementedInIRODSServer, self).__init__(
+            feature_description + ": Not supported by the connected iRODS server."
+        )
+        self.required_iRODS_version = required_iRODS_version
+
+    def __str__(self):
+        nv = self.required_iRODS_version
+        return "{}{}".format(
+            self.args,
+            " [requires iRODS version: {nv}]".format(**locals()) if nv else "",
+        )
+
+    def __repr__(self):
+        return self.__class__.__name__ + str(self)
+
+
 class iRODSExceptionMeta(type):
     codes = {}
+    positive_code_error_message = (
+        "For {name}, a positive code of {attrs[code]} was declared."
+    )
 
     def __init__(self, name, bases, attrs):
-        if 'code' in attrs:
-            iRODSExceptionMeta.codes[attrs['code']] = self
+        if "code" in attrs:
+            if attrs["code"] > 0:
+                print(
+                    self.positive_code_error_message.format(**locals()), file=sys.stderr
+                )
+                exit(1)
+            iRODSExceptionMeta.codes[attrs["code"]] = self
 
 
-class iRODSException(six.with_metaclass(iRODSExceptionMeta, Exception)):
-    pass
+class Errno:
+    """Encapsulates an integer error code from the operating system
+    and provides a text representation.
+    """
+
+    def __init__(self, arg0, *_):
+        """Initializes an object with an integer error code arg0.
+        Further arguments are optional and ignored.
+        """
+        self.int_code = arg0
+
+    def __repr__(self):
+        e = self.int_code
+        try:
+            return self.__class__.__name__ + repr(
+                tuple([e, errno.errorcode[e], os.strerror(e)])
+            )
+        except:
+            # The errno code is unrecognized, so fall through to default representation.
+            pass
+        return self.__class__.__name__ + repr(
+            tuple(
+                [
+                    e,
+                ]
+            )
+        )
+
+    def __int__(self):
+        return self.int_code
+
+
+class iRODSException(Exception, metaclass=iRODSExceptionMeta):
+    """An exception that originates from a server error.
+    Exception classes that are derived from this base and represent a concrete error, should
+    store a unique error code (X*1000) in their 'code' attribute, where X < 0.
+    """
+
+    def __init__(self, *arg):
+        explicit_errno = None
+        argl = list(arg)
+
+        # For consistency with the text representation, allow initialization with an Errno object
+        # at the end of the argument list.
+        # Example: err = UNIX_FILE_OPEN_ERR('message', Errno(13))
+        #          err_copy = eval(repr(err))
+        if hasattr(self.__class__, "code"):
+            if argl and isinstance(argl[-1], Errno):
+                explicit_errno = argl.pop()
+
+        super(iRODSException, self).__init__(*argl)
+
+        # To properly represent the Errno instance, the "code" attribute should be (-errno) plus the thousands
+        # attribute stored in the class's code attribute.  Because Python honors the instance "code" attribute
+        # over the class "code" attribute, the following does _not_ modify the class:
+        if explicit_errno:
+            self.code -= int(explicit_errno)
+
+    def __repr__(self):
+        args = tuple(self.args)
+        code = getattr(self, "code", None)
+        if code is not None:
+            os_err = abs(code) % 1000
+            if os_err:
+                args += (Errno(os_err),)
+        return self.__class__.__name__ + repr(args)
+
+
+def nominal_code(the_code, THRESHOLD=1000):
+    nominal = []
+    c = rounded_code(the_code, nominal_int_repo=nominal)
+    negated = -abs(nominal[0])
+    return (
+        c if (negated <= -abs(THRESHOLD)) else negated
+    )  # produce a negative for nonzero integer input
+
+
+def rounded_code(the_code, nominal_int_repo=()):
+    nom_err = None
+    try:
+        if isinstance(the_code, type) and issubclass(the_code, iRODSException):
+            the_code = getattr(the_code, "code", the_code)
+        if isinstance(the_code, str):
+            nom_err = globals()[the_code].code
+            return nom_err
+        elif isinstance(the_code, numbers.Integral):
+            nom_err = the_code
+            return 1000 * ((-abs(the_code) - 1) // 1000 + 1)
+        else:
+            message = "Supplied code {the_code!r} must be integer or string".format(
+                **locals()
+            )
+            raise RuntimeError(message)
+    finally:
+        if nom_err is not None and isinstance(nominal_int_repo, list):
+            nominal_int_repo[:] = [nom_err]
+
+
+def get_exception_class_by_code(code, name_only=False):
+    rounded = rounded_code(code)  # rounded up to -1000 if code <= -1000
+    cls = iRODSExceptionMeta.codes.get(rounded)
+    return (
+        cls
+        if not name_only
+        else (cls.__name__ if cls is not None else "Unknown_iRODS_error")
+    )
 
 
 def get_exception_by_code(code, message=None):
-    return iRODSExceptionMeta.codes[code](message)
+    exc_class = iRODSExceptionMeta.codes[rounded_code(code)]
+    exc_instance = exc_class(message)
+    exc_instance.code = code
+    return exc_instance
+
+
+class UnknowniRODSError(iRODSException):
+    code = 0  # covers rounded_code (errcode) if 0 > errcode > -1000
 
 
 class SystemException(iRODSException):
@@ -504,6 +657,30 @@ class SYS_RESC_QUOTA_EXCEEDED(SystemException):
     code = -110000
 
 
+class SYS_INVALID_INPUT_PARAM(SystemException):
+    code = -130000
+
+
+class SYS_BAD_INPUT(iRODSException):
+    code = -158000
+
+
+class SYS_REPLICA_DOES_NOT_EXIST(iRODSException):
+    code = -164000
+
+
+class SYS_LIBRARY_ERROR(iRODSException):
+    code = -167000
+
+
+class SYS_REPLICA_INACCESSIBLE(iRODSException):
+    code = -168000
+
+
+class SYS_NOT_ALLOWED(iRODSException):
+    code = -169000
+
+
 class UserInputException(iRODSException):
     pass
 
@@ -696,6 +873,22 @@ class NO_LOCAL_FILE_RSYNC_IN_MSI(UserInputException):
     code = -356000
 
 
+class OBJ_PATH_DOES_NOT_EXIST(iRODSException):
+    code = -358000
+
+
+class LOCKED_DATA_OBJECT_ACCESS(SystemException):
+    code = -406000
+
+
+class USER_INCOMPATIBLE_PARAMS(iRODSException):
+    code = -402000
+
+
+class CHECK_VERIFICATION_RESULTS(SystemException):
+    code = -407000
+
+
 class FileDriverException(iRODSException):
     pass
 
@@ -996,6 +1189,10 @@ class CATALOG_ALREADY_HAS_ITEM_BY_THAT_NAME(CatalogLibraryException):
     code = -809000
 
 
+class CAT_NO_CHECKSUM_FOR_REPLICA(CatalogLibraryException):
+    code = -862000
+
+
 class CAT_INVALID_RESOURCE_TYPE(CatalogLibraryException):
     code = -810000
 
@@ -1132,6 +1329,10 @@ class CAT_UNKNOWN_SPECIFIC_QUERY(CatalogLibraryException):
     code = -853000
 
 
+class CAT_STATEMENT_TABLE_FULL(CatalogLibraryException):
+    code = -860000
+
+
 class RDSException(iRODSException):
     pass
 
@@ -1176,6 +1377,22 @@ class RDA_NAME_NOT_FOUND(RDSException):
     code = -889000
 
 
+class TicketException(CatalogLibraryException):
+    pass
+
+
+class CAT_TICKET_INVALID(TicketException):
+    code = -890000
+
+
+class CAT_TICKET_EXPIRED(TicketException):
+    code = -891000
+
+
+class CAT_TICKET_USES_EXCEEDED(TicketException):
+    code = -892000
+
+
 class MiscException(iRODSException):
     pass
 
@@ -1492,6 +1709,14 @@ class DATAACCESSINX_EMPTY_IN_STRUCT_ERR(RuleEngineException):
     code = -1016000
 
 
+class RE_RUNTIME_ERROR(RuleEngineException):
+    code = -1205000
+
+
+class RE_TYPE_ERROR(RuleEngineException):
+    code = -1230000
+
+
 class NO_RULE_FOUND_ERR(RuleEngineException):
     code = -1017000
 
@@ -1860,6 +2085,10 @@ class MSI_OPERATION_NOT_ALLOWED(RuleEngineException):
     code = -1110000
 
 
+class RULE_ENGINE_ERROR(RuleEngineException):
+    code = -1828000
+
+
 class PHPException(iRODSException):
     pass
 
@@ -1874,3 +2103,27 @@ class PHP_REQUEST_STARTUP_ERR(PHPException):
 
 class PHP_OPEN_SCRIPT_FILE_ERR(PHPException):
     code = -1602000
+
+
+class DIRECT_CHILD_ACCESS(iRODSException):
+    code = -1816000
+
+
+class PAMException(iRODSException):
+    pass
+
+
+class PAM_AUTH_NOT_BUILT_INTO_CLIENT(PAMException):
+    code = -991000
+
+
+class PAM_AUTH_NOT_BUILT_INTO_SERVER(PAMException):
+    code = -992000
+
+
+class PAM_AUTH_PASSWORD_FAILED(PAMException):
+    code = -993000
+
+
+class PAM_AUTH_PASSWORD_INVALID_TTL(PAMException):
+    code = -994000
